/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __PXE_UNDI_H__
#define __PXE_UNDI_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE data types */
#include "pxe_types.h"

/*	Parameter structure and type definitions for TFTP API version 2.x
 *
 *	PXENV.H needs to be #included before this file.
 *
 *	All of the UNDI API services are still available after the stack
 *	has been unloaded.
 */

/* PXE UNDI API function numbers.
 */
 
/* Constants that define the offsets into the table of functions.
 * One of these must be loaded into the BX register before making
 * a call a UNDI API service. 
 */
#define PXENV_START_UNDI		0x0000
#define PXENV_UNDI_STARTUP	 	0x0001
#define PXENV_UNDI_CLEANUP	 	0x0002
#define PXENV_UNDI_INITIALIZE 		0x0003
#define PXENV_UNDI_RESET_NIC		0x0004
#define PXENV_UNDI_SHUTDOWN		0x0005
#define PXENV_UNDI_OPEN			0x0006
#define PXENV_UNDI_CLOSE		0x0007
#define PXENV_UNDI_TRANSMIT		0x0008
#define PXENV_UNDI_SET_MCAST_ADDR	0x0009
#define PXENV_UNDI_SET_STATION_ADDR	0x000A
#define PXENV_UNDI_SET_PACKET_FILTER	0x000B
#define PXENV_UNDI_GET_INFORMATION	0x000C
#define PXENV_UNDI_GET_STATISTICS	0x000D
#define PXENV_UNDI_CLEAR_STATISTICS	0x000E
#define PXENV_UNDI_INITIATE_DIAGS	0x000F
#define PXENV_UNDI_FORCE_INTERRUPT	0x0010
#define PXENV_UNDI_GET_MCAST_ADDR	0x0011
#define PXENV_UNDI_GET_NIC_TYPE		0x0012
#define PXENV_UNDI_GET_NDIS_INFO	0x0013
#define PXENV_UNDI_ISR			0x0014
#define PXENV_STOP_UNDI			0x0015
#define PXENV_UNDI_GET_STATE		0x0016

/* This is used to set the mutlicast address within the other data
 * structures.
 * paramters:
 *     addr_count - the number of addresses supplied.
 *     mcast_addr - the multicast addresses.
 */
#define ADDR_LEN 16             /* the max length of each address */
#define MAXNUM_MCADDR 8		/* the max number of addresses that can 
                                 supplied */ 
typedef struct pxenv_undi_mcast_addr_s {
    u_int16_t addr_count;
    u_int8_t MCastAddr[MAXNUM_MCADDR][ADDR_LEN];
} pxenv_undi_mcast_addr_t;


/* Data structure for a transmit block descriptor.
 * parameters:
 *     length - the length of the data buffer in bytes.
 *     xmit_buffer -  the buffer address in offset:segment format.
 *     datablk_count - number of data blocks.
 *     datablk.type - this is the type of the Valid values are:
 *                 0 - 32 bit phys pointer in tddataptr, not supported in this
 *		       version of lsa
 *		   1 - seg:offset in tddataptr which can be a real mode or
 *                     16-bit protected mode pointer .
 *	datablk.byte - reserved, must be zero.
 *	datablk.len - data block length in bytes.
 *	datablk.buffer - 32 bit pointer to data buffer.
 */
#define	MAX_DATA_BLKS 8
typedef	struct s_pxenv_undi_tbd_s {
    u_int16_t length;
    addr32_t *xmit_buffer;
    u_int16_t datablk_count;
    struct datablk {
	u_int8_t type;
	u_int8_t byte;
	u_int16_t len;
	u_int32_t buffer;
    } data_block[MAX_DATA_BLKS];
} pxenv_undi_tbd_t;

/* Data structure for pxenv_undi_startup().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_startup_s {
    pxenv_status_t status;
} pxenv_undi_startup_t;

/* Data structure for pxenv_undi_cleanup().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_cleanup_s {
    pxenv_status_t status;
} pxenv_undi_cleanup_t;

/* Data structure for pxenv_undi_initialize().
 * parameters:
 *     protocolini - this is the 32 bit address of the memory copy of
 *                   the driver module in the protocol.ini file. This
 *                   used to pass into to the ndis driver.
 *     reserved - FIXME: what is this ?
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_initialize_s {
    pxenv_status_t status;
    u_int32_t protocolini;
    u_int8_t reserved[8];
} pxenv_undi_initialize_t;

/* Data structure for pxenv_undi_reset_nic().
 * parameters:
 *     mcast_buf - The multicast address is not stored anywhere, so when
 *                 a client application resets the adaptor, this must be
 *                 provided.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_reset_nic_s {
    pxenv_status_t status;
    pxenv_undi_mcast_addr_t mcast_buf; /* multicast address list */
					/* see note below  */
} pxenv_undi_reset_nic_t;

/* Data structure for pxenv_undi_shutdown().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_shutdown_s {
    pxenv_status_t status;
} pxenv_undi_shutdown_t;

/* Data structure for pxenv_undi_open().
 * parameters:
 *     open_flag - this is an adaptor specific flag used to pass the
 *                 Open flags from the protocol driver. Can be zero.
 *     pkt_filter - one of the constants below (can be ORed together).
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
#define FLTR_DIRECTED 0x01  	/* directed/multicast */
#define FLTR_BRDCST   0x02  	/* broadcast packets */
#define FLTR_PRMSCS   0x04  	/* any packet on the net */
#define FLTR_SRC_RTG  0x08  	/* source routing packet */
typedef struct pxenv_undi_open_s {
    pxenv_status_t status;
    u_int16_t open_flag;
    u_int16_t pkt_filter;
    pxenv_undi_mcast_addr_t mcastbuffer;
} pxenv_undi_open_t;

/* Data structure for pxenv_undi_close().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_close_s {
    pxenv_status_t status;
} pxenv_undi_close_t;

/* Data structure for pxenv_endi_transmit().
 * parameters:
 *     protocol - the protocol of the upper layer that is making this
 *                function call. If the upper layer has filled in 
 *                the media header, this must be zero.
 *     xmit_flags - if zero, use the dest_addr. If 1, the adapator
 *                 supplies the broadcast address.
 *     dest_addr - the hardware address of the destination media in
 *                 offset:segment format. If this isn't known, this
 *                 field can be zero, and xmit_flags must be
 *                 XMT_BROADCAST.
 *     tdb_addr - the transmit buffer in offset:segment format.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
#define P_UNKNOWN 0                     /* flags for xmit_flag */
#define P_IP	  1
#define P_ARP	  2
#define P_RARP	  3

#define XMT_DESTADDR	0x0000		/* destination address given */
#define XMT_BROADCAST	0x0001		/* use broadcast address */
typedef struct pxenv_undi_transmit_s {
    pxenv_status_t status;
    u_int8_t protocol;
    u_int8_t xmit_flags;
    u_int8_t dest_addr[6];
    pxenv_undi_tbd_t *tbd_addr;
    u_int32_t reserved[2];
} pxenv_undi_transmit_t;

/* Data structure for pxenv_undi_mcast_addr().
 * parameters:
 *     mcast_buffer - the multicast address.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_set_mcast_addr_s {
    pxenv_status_t status;
    pxenv_undi_mcast_addr_t mcast_buffer;
} pxenv_undi_set_mcast_addr_t;

/* Data structure for pxenv_undi_set_station_addr().
 * parameters:
 *     station_addr - the MAC address to be used.
 * returns:
 *     status - the exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_set_station_addr_s {
    pxenv_status_t status;
    mac_addr_t station_addr;
} pxenv_undi_set_station_addr_t;

/* Data structure for pxenv_undi_set_packet_filter().
 * parameters:
 *     filter - the new filter for the NIC receive unit. Permissable
 *              values are FLTR_DIRECTED, FLTR_BRDCST, FLTR_PRMSCS,
 *              FLTR_SRC_RTG.
 * returns:
 *     status - the exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_set_packet_filter_s {
    pxenv_status_t status;
    u_int8_t filter;
} pxenv_undi_set_packet_filter_t;

/* Data structure for pxenv_undi_startup().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     baseio - the base I/O address of the adpator.
 *     irqnum - the IRQ number of the adaptor.
 *     maxmtu - the maxium MTUs. FIXME:
 *     hwtype - the protocol type at the hardware level. There is an
 *              RFRC1010 for "Assigned Numbers". As this number may
 *              not be validated by the application and hence adding
 *              new numbers to the list should be fine at any time.
 *     hw_addrlen - the length of the hardware address.
 *     current_node_addr - the current hardware address.
 *     perm_node_addr - the permanent hardware address.
 *     rom_addr - the ROM address.
 *     rxbuflen - the length of the receive buffer in bytes.
 *     txbuflen - the length of the transmit buffer in bytes.
 */
#define ETHER_TYPE	1       /* the hardware types for hwtype */
#define EXP_ETHER_TYPE	2
#define IEEE_TYPE	6
#define ARCNET_TYPE	7
typedef struct pxenv_undi_get_information_s {
    pxenv_status_t status;
    u_int16_t baseio;
    u_int16_t irqnum;
    u_int16_t maxmtu;
    u_int16_t hwtype;
    u_int16_t hw_addrlen;
    u_int8_t current_node_addr[ADDR_LEN];
    u_int8_t perm_node_addr[ADDR_LEN];
    u_int16_t rom_addr;
    u_int16_t rxbuflen;
    u_int16_t txbuflen;
} pxenv_undi_get_information_t;

/* Data structure for pxenv_undi_get_statistics().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     tx_good_frames - the count of good transmissions.
 *     rx_good_frames - the count of good receptions.
 *     rx_crc_err - the count of frames with CRC errors.
 *     rx_discard_err - the count of rames discarded.
 */
typedef struct pxenv_undi_get_statistics_s {
    pxenv_status_t status;
    u_int32_t tx_good_frames;
    u_int32_t rx_good_frames;
    u_int32_t rx_crc_err;
    u_int32_t rx_discard_err;
} pxenv_undi_get_statistics_t;

/* Data structure for pxenv_undi_clear_statistics().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_clear_statistics_s {
    pxenv_status_t status;
} pxenv_undi_clear_statistics_t;

/* Data structure for pxenv_undi_initiate_diags().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_initiate_diags_s {
    pxenv_status_t status;
} pxenv_undi_initiate_diags_t;

/* Data structure for pxenv_undi_force_interupts().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_undi_force_interrupt_s {
    pxenv_status_t status;
} pxenv_undi_force_interrupt_t;


/* Data structure for pxenv_undi_get_mcast_addr().
 * parameters:
 *     ip_addr - the multicast IP number.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     mcast_addr - the hardware multicast address.
 */
typedef struct pxenv_undi_get_mcast_addr_s {
    pxenv_status_t status;
    ip4_t ip_ddr;
    mac_addr_t mcast_addr;
} pxenv_undi_get_mcast_addr_t;


/* Data structure for pxenv_undi_get_nic_type().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     nictype - This is set to 0x2 if a PCI card, or 0x3 if PNP
 *   There is one of two data structures filled in, depending wether
 *   it's a PCI or a PNP adaptor card.
 *     pci.vendor_id - the vendor ID.
 *     pci.dev_id - the device ID.
 *     pci.base_class - the base class of the device.
 *     pci.sub_class - the sub class of the device.
 *     pci.prog_inf - the program interface.
 *     pci.rev - the revision number.
 *     pci.busdevfunc - bus device and function numbers.
 *     pci.subvendor_id - the sub vendor ID.
 *     pci.subdevice_id - the sub device ID.
 *   or
 *     pnp.eisa_dev_id - the EISA device ID.
 *     pnp.base_class - the base class of the device.
 *     pnp.sub_class - the sub class of the device.
 *     pnp.prog_intf - the program interface.
 *     pnp.cardselnum - the card selector number 
 *     pnp.reserved - data structure padding to make this aligned.
 */
typedef struct pxenv_undi_get_nic_type_s {
    pxenv_status_t status;
    u_int8_t nictype;
    union {
	struct {
	    u_int16_t vendor_id;
	    u_int16_t dev_id;
	    u_int8_t base_class;
	    u_int8_t sub_class;
	    u_int8_t prog_intf;
	    u_int8_t rev;
	    u_int16_t busdevfunc;
	    u_int16_t subvendor_id;
	    u_int16_t subdevice_id;
	} pci;
	struct {
	    u_int32_t eisa_dev_id;
	    u_int8_t base_class;
	    u_int8_t sub_class;
	    u_int8_t prog_intf;
	    u_int16_t cardselnum;
	    u_int8_t reserved;
	} pnp;
    } pci_pnp_info;
} pxenv_undi_get_nic_type_t;

/* Data structure for pxenv_undi_get_iface_info().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     iface_type - this is the type of interface in ASCIIZ format.
 *     link_speed - as defined in the NDIS 2.0x specification.
 *     service_flags - as defined in the NDIS 2.0x specification.
 *     reserved - must be set to all zeros as it's undefined for now.
 */
typedef struct pxenv_undi_get_iface_info_s {
    pxenv_status_t status;
    u_int8_t iface_type[16];
    u_int32_t link_speed;
    u_int32_t service_flags;
    u_int32_t reserved[4];
} pxenv_undi_get_iface_info_t;

/* Data structure for pxenv_undi_isr().
 * parameters:
 *     func_flag - this is set to one of the PXENV_UNDI_ISR_IN_*
 *              constants.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     func_flag - this is set to one of the PXENV_UNDI_ISR_OUT_*
 *              constants.
 *     buffer_length - the length of the buffer in bytes.
 *     frame_length - the length of the frame in bytes.
 *     frame_header_length - the length of the frame header in bytes.
 *     frame_addr - the frame address in offset:segment format.
 *     prot_type - the type of the protocol. Valid values are:
 *                 1 - PXENV_UNDI_PROTO_IP
 *                 2 - PXENV_UNDI_PROTO_ARP
 *                 3 - PXENV_UNDI_PROTO_RARP
 *                 4 - other
 *     pkt_type - the type of packet. Valid values are:
 */
#define PXENV_PACKET_TYPE_DHCP_DISCOVER  1
#define PXENV_PACKET_TYPE_DHCP_ACK       2
#define PXENV_PACKET_TYPE_BINL_REPLY     3

typedef struct pxenv_undi_isr_s {
    pxenv_status_t status;
    u_int16_t func_flag;
    u_int16_t buffer_length;
    u_int16_t frame_length;
    u_int16_t frame_header_length;
    addr32_t *frame_addr;
    u_int8_t  prot_type;
    u_int8_t  pkt_type;
} pxenv_undi_isr_t;

/* Possible values for the protocol type */
#define PXENV_UNDI_PROTO_IP 1
#define PXENV_UNDI_PROTO_ARP 2
#define PXENV_UNDI_PROTO_RARP 3

#define PXENV_UNDI_ISR_IN_START 1
/* Call to start processing one of our interrupts. */
#define PXENV_UNDI_ISR_IN_PROCESS 2
/* Call to start/continue receiving data from receive buffer(s). */
#define PXENV_UNDI_ISR_IN_GET_NEXT 3

/* Possible responses from PXENV_UNDI_ISR_IN_START */

/* This is our interrupt.  Deal with it. */
#define PXENV_UNDI_ISR_OUT_OURS	0
/* This is not our interrupt. Ignore it. */
#define PXENV_UNDI_USR_OUT_NOT_OURS 1

/* Possible responses from PXENV_UNDI_ISR_IN_PROCESS and
 * PXENV_UNDI_ISR_IN_PROCESS
 */

/* We are done processing this interrupt. */
#define PXENV_UNDI_ISR_OUT_DONE	0
/* We completed a transmit interrupt. */
#define PXENV_UNDI_ISR_OUT_TRANSMIT 2
/* Get data from receive buffer. */
#define PXENV_UNDI_ISR_OUT_RECEIVE 3
/* We don't know what this is */
#define PXENV_UNDI_ISR_OUT_BUSY	4

/* Data structure for pxenv_undi_get_state().
 * parameters:
 *     none
 * returns:
 *     status - the exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     undi_state - the state of the UNDI engine.
 */
typedef struct pxenv_undi_get_state_s {
    pxenv_status_t status;
    u_int16_t undi_state;
} pxenv_undi_get_state_t;


/* Prototypes for the UNDI PXE API functions */
pxenv_ret_code_t
pxenv_undi_startup (pxenv_undi_startup_t *undistartup);
pxenv_ret_code_t
pxenv_undi_cleanup (pxenv_undi_cleanup_t *undicleanup);
pxenv_ret_code_t
pxenv_undi_initialize (pxenv_undi_initialize_t *undiinit);
pxenv_ret_code_t
pxenv_undi_reset_nic (pxenv_undi_reset_nic_t *undireset);
pxenv_ret_code_t
pxenv_undi_shutdown (pxenv_undi_shutdown_t *undishut);
pxenv_ret_code_t
pxenv_undi_open (pxenv_undi_open_t *undiopen);
pxenv_ret_code_t
pxenv_undi_close (pxenv_undi_close_t *undiclose);
pxenv_ret_code_t
pxenv_undi_transmit (pxenv_undi_transmit_t *unditrans);
pxenv_ret_code_t
pxenv_undi_set_mcast_addr (pxenv_undi_set_mcast_addr_t *undimcast);
pxenv_ret_code_t
pxenv_undi_set_station_addr (pxenv_undi_set_station_addr_t *undistat);
pxenv_ret_code_t
pxenv_undi_set_packet_filter (pxenv_undi_set_packet_filter_t *undifilter);
pxenv_ret_code_t
pxenv_undi_get_information (pxenv_undi_get_information_t *undiinfo);
pxenv_ret_code_t
pxenv_undi_get_statistics (pxenv_undi_get_statistics_t *undistats);
pxenv_ret_code_t
pxenv_undi_clear_statistics (pxenv_undi_clear_statistics_t *undiclrstats);
pxenv_ret_code_t
pxenv_undi_initiate_diags (pxenv_undi_initiate_diags_t *undidiag);
pxenv_ret_code_t
pxenv_undi_force_interrupt (pxenv_undi_force_interrupt_t *undiint);
pxenv_ret_code_t
pxenv_undi_get_mcast_addr (pxenv_undi_get_mcast_addr_t *undimcast);
pxenv_ret_code_t
pxenv_undi_get_nic_type (pxenv_undi_get_nic_type_t *undinic);
pxenv_ret_code_t
pxenv_undi_get_iface_info (pxenv_undi_get_iface_info_t *undiiface);
pxenv_ret_code_t
pxenv_undi_get_state (pxenv_undi_get_state_t *undistate);
pxenv_ret_code_t
pxenv_undi_isr (pxenv_undi_isr_t *undiisr);

#endif /* __PXE_UNDI_H__ */
