/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __TFTP_H__
#define __TFTP_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* IP port for the TFTP server */
#define	TFTP_SERVER_PORT 69
#define	TFTP_CLIENT_PORT 2000

/* IP port for the MTFTP server used for Intel's PXE */
#define	MTFTP_SERVER_PORT 75
#define	MTFTP_CLIENT_PORT 76

#define	TFTP_DEFAULTSIZE_PACKET	512
#define	TFTP_MAX_PACKET	1432

/* We define these based on what's in arpa/tftp.h. We just like our
 names better, cause they're clearer */
#define TFTP_RRQ 1
#define TFTP_WRQ 2
#define TFTP_DATA 3
#define TFTP_ACK 4
#define TFTP_ERROR 5
#define TFTP_OACK 6

#define TFTP_CODE_EOF 1
#define TFTP_CODE_MORE 2
#define TFTP_CODE_ERROR 3
#define TFTP_CODE_BOOT 4
#define TFTP_CODE_CFG 5

#define TFTP_EUNDEF 0                   /* not defined */
#define TFTP_ENOTFOUND 1                /* file not found */
#define TFTP_EACCESS 2                  /* access violation */
#define TFTP_ENOSPACE 3                 /* disk full or allocation exceeded */
#define TFTP_EBADOP 4                   /* illegal TFTP operation */
#define TFTP_EBADID 5                   /* unknown transfer ID */
#define TFTP_EEXISTS 6                  /* file already exists */
#define TFTP_ENOUSER 7                  /* no such user */


#define	TFTP_DEFAULT_FILENAME	"kernel"

/* There is a struct tftphdr in arpa/tftp.h, but we define our
 * own here because this is cleaner, and maps to the same data layout.
 */
struct tftp {
    u_int16_t opcode;
    union {
        int8_t rrq[TFTP_DEFAULTSIZE_PACKET];
        struct {
            u_int16_t block;
            int8_t download[TFTP_MAX_PACKET];
        } data;
        struct {
            u_int16_t block;
        } ack;
        struct {
            u_int16_t errcode;
            int8_t errmsg[TFTP_DEFAULTSIZE_PACKET];
        } err;
        struct {
            int8_t data[TFTP_DEFAULTSIZE_PACKET+2];
        } oack;
    } u;
};
typedef struct tftp tftp_t;

extern int32_t blocksize;

//extern int tftp_decode_packet(tftp_t *pkt, oskit_size_t pkt_size);

extern int
fill_tftp_request(tftp_t *, u_int16_t opcode,
                  const char *filename, int32_t blockno);

#endif /* __TFTP_H__ */

